package net.keil.openFileView.preferences;

import org.eclipse.jface.preference.*;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;
import net.keil.openFileView.OpenFileViewPlugin;
import net.keil.openFileView.Resources;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 * 
 * @author Torsten Keil
 * @since 1.4.0
 */

public class OpenFileViewPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public OpenFileViewPreferencePage() {
		super(GRID);
		setPreferenceStore(OpenFileViewPlugin.getDefault().getPreferenceStore());
		setDescription(Resources.getString("Preferences.Description"));
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		
		addField(
				new BooleanFieldEditor(
					PreferenceConstants.P_SHOW_CLOSE_COLUMN,
					Resources.getString("Preferences.ShowCloseColumn"),
					getFieldEditorParent()));

		addField(
			new BooleanFieldEditor(
				PreferenceConstants.P_SHOW_FILE_ICONS,
				Resources.getString("Preferences.ShowFileIcons"),
				getFieldEditorParent()));

		addField(
				new BooleanFieldEditor(
					PreferenceConstants.P_SHOW_PROJECT_ICONS,
					Resources.getString("Preferences.ShowProjectIcons"),
					getFieldEditorParent()));

		if (OpenFileViewPlugin.getDefault().using31) {
			addField(new RadioGroupFieldEditor(
					PreferenceConstants.P_USE_31_LOOK,
					Resources.getString("Preferences.FontLookTitle"),
					1,
					new String[][] { 
							{ Resources.getString("Preferences.O_30_Look"), PreferenceConstants.O_30_LOOK }, 
							{	Resources.getString("Preferences.O_31_Look"), PreferenceConstants.O_31_LOOK }
					}, getFieldEditorParent()));
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}
	
}