package net.keil.openFileView.util;

import net.keil.openFileView.OpenFileViewPlugin;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;

/**
 * This class represents an item in the OpenFileView table. <br>
 * Uses lazy-loading - attributes are retrieved and stored at first access. 
 * 
 * @author Torsten Keil
 * @since 1.2.0
 */
public class OpenFileItem {
	
	private IEditorReference editorRef;
	private IEditorPart editorPart;
	private IFile file;
	private String fileName;
	private Image fileIcon;
	private String path;
	private IProject project;
	private String projectName;
	private Image projectIcon;
	
	public OpenFileItem(IEditorReference edRef) {
		this.editorRef = edRef;
	}

	/**
	 * Gets the EditorPart for the given item.<br>
	 * Tries to restore the editor.
	 * 
	 * @return Returns the editorPart.
	 */
	public IEditorPart getEditorPart() {
		if (editorPart == null) {
			editorPart = (IEditorPart) editorRef.getPart(true);
		}
		return editorPart;
	}

	/**
	 * Gets the EditorPart for the given item.<br>
	 * Tries to restore the editor, when restore is true.
	 * 
	 * @return Returns the editorPart.
	 */
	public IEditorPart getEditorPart(boolean restore) {
		if (editorPart == null) {
			editorPart = (IEditorPart) editorRef.getPart(restore);
		}
		return editorPart;
	}
	
	/**
	 * @return Returns the editorRef.
	 */
	public IEditorReference getEditorRef() {
		return editorRef;
	}

	/**
	 * @return Returns the fileIcon.
	 */
	public Image getFileIcon() {
		if (fileIcon == null) {
			fileIcon = editorRef.getTitleImage();
		}
		return this.fileIcon;
	}

	/**
	 * @return Returns the fileName.
	 */
	public String getFileName() {
		if (fileName == null) {
			fileName = editorRef.getTitle();
		}
		return fileName;
	}

	/**
	 * @return Returns the path.
	 */
	public String getPath() {
		return path;
	}

	/**
	 * @return Returns the projectName.
	 */
	public String getProjectName() {
		if (projectName == null) {
			IProject proj = getProject();
			projectName = proj != null ? proj.getName() : "";
		}
		return projectName;
	}

	/**
	 * @return Returns the projectIcon.
	 */
	public Image getProjectIcon() {
		return projectIcon;
	}

	/**
	 * @return Returns the project.
	 */
	public IProject getProject() {
		if (project == null) {
			IFile file = getFile();
			IResource res = file;
			project = res != null ? res.getProject() : null;
		}
		return project;
	}

	/**
	 * @return Returns the file.
	 */
	public IFile getFile() {
		if (file == null) {
			file = OpenFileViewPlugin.getDefault().getFileForReference(editorRef);
		}
		return file;
	}
	

}
