package net.keil.openFileView.views;


import java.util.Hashtable;

import net.keil.openFileView.OpenFileViewPlugin;
import net.keil.openFileView.Resources;
import net.keil.openFileView.preferences.PreferenceConstants;
import net.keil.openFileView.util.OpenFileItem;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;


/**
 * @author Torsten Keil
 * @since 1.0.0
 */
public class OpenFileView extends ViewPart {
	
	private TableViewer viewer;
	private Action closeEditorAction;
	private Action selectEditorAction;
	private OpenFilesChangeListener listener;
	
	IEditorReference activeEditorRef;
	
	private Hashtable cache;
	
	Font defaultFont;
	Font boldFont;
	
	Color defaultColor;
	Color hoverColor;
	
	private boolean isInitialized = false;
	
	private TableColumn closeColumn;
	private TableColumn fileColumn;
	private TableColumn projectColumn;
	
	private OpenFilesNameSorter sorter;
	
	private IMemento memento;
	
	Image closeImg;
	Image closeImgThin;

	Image noArrow;
	Image upArrow;
	Image downArrow;
	Image[] tableIcons;

	/**
	 * The constructor.
	 */
	public OpenFileView() {
		// *** creating fonts ***
		defaultFont = JFaceResources.getDefaultFont();
    FontData fontData[] = defaultFont.getFontData();
    // Adding the bold attribute
    for (int i = 0; i < fontData.length; i++)
        fontData[i].setStyle(fontData[i].getStyle() | SWT.BOLD);
    boldFont = new Font(null, fontData);
    
    // *** creating colors
    defaultColor = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GRAY);
    hoverColor = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
    
    // *** creating Images ***
    ImageDescriptor id_close = ImageDescriptor.createFromFile(OpenFileView.class, "/icons/close_view.gif");
    closeImg = id_close.createImage();
    ImageDescriptor id_close_thin = ImageDescriptor.createFromFile(OpenFileView.class, "/icons/thin_close_view.gif");
    closeImgThin = id_close_thin.createImage();
    ImageDescriptor id_no = ImageDescriptor.createFromFile(OpenFileView.class, "/icons/Arrow_no.gif");
    noArrow = id_no.createImage();
    ImageDescriptor id_down = ImageDescriptor.createFromFile(OpenFileView.class, "/icons/Arrow_up.gif");
    downArrow = id_down.createImage();
    ImageDescriptor id_up = ImageDescriptor.createFromFile(OpenFileView.class, "/icons/Arrow_down.gif");
    upArrow = id_up.createImage();
    tableIcons = new Image[] {noArrow, upArrow, downArrow};
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	public void dispose() {
		super.dispose();
		// remove listeners
		getSite().getPage().removePartListener(this.listener);
		getSite().getWorkbenchWindow().removePerspectiveListener(this.listener);
		IEditorReference[] editorReferences = getSite().getPage().getEditorReferences();
		for (int i = 0; i < editorReferences.length; i++) {
			IEditorReference reference = editorReferences[i];
			reference.removePropertyListener(this.listener);
		}
		IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
		store.removePropertyChangeListener(this.listener);
		// empty cache
		cache = null;
		// dispose ressources
		// *** fonts ***
		boldFont.dispose();
		// *** colors ***
		defaultColor.dispose();
		hoverColor.dispose();
		// *** images ***
		closeImg.dispose();
		closeImgThin.dispose();
		noArrow.dispose();
		downArrow.dispose();
		upArrow.dispose();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite, org.eclipse.ui.IMemento)
	 */
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		super.init(site, memento);
		this.memento = memento;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.ViewPart#saveState(org.eclipse.ui.IMemento)
	 */
	public void saveState(IMemento memento) {
		super.saveState(memento);
		TableColumn[] columns = viewer.getTable().getColumns();
		for (int i = 0; i < columns.length; i++) {
			TableColumn column = columns[i];
			String text = column.getText();
			int width = column.getWidth();
			memento.putInteger("COL_" + text + "_WIDTH", width);
		}
		sorter.saveState(memento);
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		listener = new OpenFilesChangeListener(this);
		viewer = new TableViewer(parent, SWT.MULTI | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		Table table = viewer.getTable();
		closeColumn = new TableColumn(table, SWT.LEAD);
		closeColumn.setText("");
		closeColumn.setWidth(16);
		closeColumn.setResizable(false);
		fileColumn = new TableColumn(table, SWT.LEAD);
		fileColumn.setText(Resources.getString("TableColumn.File"));
		int width = 100;
		if (memento != null && memento.getInteger("COL_" + fileColumn.getText() + "_WIDTH") != null) {
			width = memento.getInteger("COL_" + fileColumn.getText() + "_WIDTH").intValue();
		}
		fileColumn.setWidth(width);
		projectColumn = new TableColumn(table, SWT.LEAD);
		projectColumn.setText(Resources.getString("TableColumn.Project"));
		if (memento != null && memento.getInteger("COL_" + fileColumn.getText() + "_WIDTH") != null) {
			width = memento.getInteger("COL_" + projectColumn.getText() + "_WIDTH").intValue();
		}
		projectColumn.setWidth(width);
		table.setHeaderVisible(true);
		viewer.setContentProvider(new ViewContentProvider(this));
		if (OpenFileViewPlugin.getDefault().using31) {
			viewer.setLabelProvider(new ViewLabelFontProvider(this));
		} else {
			viewer.setLabelProvider(new ViewLabelProvider(this));
		}
		sorter = new OpenFilesNameSorter(this, memento);
		viewer.setSorter(sorter);
		viewer.setInput(getViewSite());
		updateTableHeaderIcons();
		initListeners(viewer);
		// generated
		makeActions();
		hookContextMenu();
		hookTableClickAction();
		isInitialized = true;
		refreshViewer();
	}
	
	protected void initListeners(TableViewer viewer) {
		getSite().getPage().addPartListener(this.listener);
		getSite().getWorkbenchWindow().addPerspectiveListener(this.listener);
		IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
		store.addPropertyChangeListener(this.listener);
	}
	
	protected void updateElements() {
		refreshViewer();
	}
	
	protected void updateElements(IEditorReference editorRef) {
		this.activeEditorRef = editorRef;
		refreshViewer();
	}
	
	protected void refreshViewer() {
		if (isInitialized) {
			IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
			if (store.getBoolean(PreferenceConstants.P_SHOW_CLOSE_COLUMN)) {
				closeColumn.setWidth(16);
			} else {
				closeColumn.setWidth(0);
			}
			viewer.getTable().redraw();
			viewer.refresh();
		}
	}
	
	protected void updateTableHeaderIcons() {
		fileColumn.setImage(tableIcons[sorter.getSortModusFile()]);
		projectColumn.setImage(tableIcons[sorter.getSortModusProject()]);
	}
	
	protected void addEditor(IEditorReference edRef) {
		OpenFileItem item = new OpenFileItem(edRef);
		getCache().put(edRef, item);
		edRef.addPropertyListener(this.listener);
		updateElements();
	}
	
	protected void removeEditor(IEditorReference edRef) {
		edRef.removePropertyListener(this.listener);
		getCache().remove(edRef);
		updateElements();
	}

	// generated

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				OpenFileView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	public void fillContextMenu(IMenuManager manager) {
		manager.add(selectEditorAction);
		manager.add(closeEditorAction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	public void makeActions() {
//		setActionGroup(new MainActionGroup(this));
		closeEditorAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object[] objects = ((IStructuredSelection)selection).toArray();
				for (int i = 0; i < objects.length; i++) {
					Object obj = objects[i];
					if (obj instanceof OpenFileItem) {
						OpenFileItem item = (OpenFileItem) obj;
						if (item != null) {
							item.getEditorRef().getPage().closeEditor(item.getEditorPart(), true);
						}
					}
					if (obj instanceof Object[]) {
						OpenFileItem item = ((OpenFileItem)((Object[])obj)[getFileColumnIndex()]);
						if (item != null) {
							item.getEditorRef().getPage().closeEditor(item.getEditorPart(), true);
						}
					}
				}
			}
		};
		closeEditorAction.setText(Resources.getString("CloseEditor.0"));
		closeEditorAction.setToolTipText(Resources.getString("CloseEditor.1"));
		
		selectEditorAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection)selection).getFirstElement();
				if (obj instanceof OpenFileItem) {
					OpenFileItem item = (OpenFileItem) obj;
					if (item != null && !item.getEditorRef().equals(activeEditorRef)) {
						item.getEditorRef().getPage().activate(item.getEditorPart());
					}
				}
				if (obj instanceof Object[]) {
					OpenFileItem item = ((OpenFileItem)((Object[])obj)[getFileColumnIndex()]);
					if (item != null) {
						item.getEditorRef().getPage().activate(item.getEditorPart());
					}
				}
			}
		};
		selectEditorAction.setText(Resources.getString("SelectEditor.0"));
		selectEditorAction.setToolTipText(Resources.getString("SelectEditor.1"));
	}

	private void hookTableClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				selectEditorAction.run();
			}
		});
    viewer.getTable().addListener(SWT.MouseDown, new Listener() {
      public void handleEvent(Event event) {
      	if (event.button != 1) return;
        Point pt = new Point(event.x, event.y);
        TableItem item = viewer.getTable().getItem(pt);
        if (item == null)
          return;
        Rectangle closeRect = item.getBounds(getCloseColumnIndex());
        if (closeRect.contains(pt)) {
        	closeEditorAction.run();
        }
      }
    });		
		TableColumn[] columns = viewer.getTable().getColumns();
		for (int i = 0; i < columns.length; i++) {
			TableColumn column = columns[i];
			column.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					// Noop
				}
				public void widgetSelected(SelectionEvent e) {
					if (e.getSource() instanceof TableColumn) {
						TableColumn col = (TableColumn) e.getSource();
						if (col.equals(fileColumn)) {
							sorter.changeSortFile();
						} else if (col.equals(projectColumn)) {
							sorter.changeSortProject();
						}
						updateTableHeaderIcons();
						refreshViewer();
					}
				}
			});
		}
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	public Hashtable getCache() {
		if (cache == null) {
			cache = new Hashtable();
			IEditorReference[] editorReferences = getSite().getPage().getEditorReferences();
			for (int i = 0; i < editorReferences.length; i++) {
				addEditor(editorReferences[i]);
			}
		}
		return cache;
	}

	/**
	 * @return Returns the closeColumnIndex.
	 */
	public int getCloseColumnIndex() {
		return viewer.getTable().indexOf(closeColumn);
	}

	/**
	 * @return Returns the fileColumnIndex.
	 */
	public int getFileColumnIndex() {
		return viewer.getTable().indexOf(fileColumn);
	}

	/**
	 * @return Returns the projectColumnIndex.
	 */
	public int getProjectColumnIndex() {
		return viewer.getTable().indexOf(projectColumn);
	}
}
