package net.keil.openFileView.views;

import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IMemento;

/**
 * Sorts the elements by name
 * 
 * @author Torsten Keil
 * @since 1.2.0
 */
class OpenFilesNameSorter extends ViewerSorter {
	
	public static final int SORT_NO = 0;
	public static final int SORT_UP = 1;
	public static final int SORT_DOWN = 2;
	
	private int fileSort = SORT_UP;
	private int projectSort = SORT_NO;
	
	private OpenFileView view;
	
	public OpenFilesNameSorter(OpenFileView view, IMemento memento) {
		super();
		this.view = view;
		init(memento);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public int compare(Viewer viewer, Object e1, Object e2) {
		String name1 = "";
		String name2 = "";
		int projColIndex = view.getProjectColumnIndex();
		int fileColIndex = view.getFileColumnIndex();
		boolean fallback = true;
		if (viewer instanceof ContentViewer) { // Default
			IBaseLabelProvider labelProvider = ((ContentViewer)viewer).getLabelProvider();
			if (labelProvider instanceof ViewLabelProvider) {
				ViewLabelProvider viewLabelProvider = (ViewLabelProvider) labelProvider;
				if (viewLabelProvider != null) {
					fallback = false;
					int compProject = 0;
					if (SORT_NO != projectSort) {
						name1 = viewLabelProvider.getColumnText(e1, projColIndex);
						name2 = viewLabelProvider.getColumnText(e2, projColIndex);
						compProject = name1.compareToIgnoreCase(name2);
						if (SORT_DOWN == projectSort) compProject *= -1;
					}
					if (SORT_NO != fileSort && compProject == 0) {
						name1 = viewLabelProvider.getColumnText(e1, fileColIndex);
						name2 = viewLabelProvider.getColumnText(e2, fileColIndex);
						int compFile = name1.compareToIgnoreCase(name2);
						if (SORT_DOWN == fileSort) compFile *= -1;
						return compFile;
					} else {
						return compProject;
					}
				}
			}
		}
		if (fallback) { // Fallback
			if (e1 instanceof IEditorReference) {
				name1 = ((IEditorReference)e1).getTitle();
			}
			if (e2 instanceof IEditorReference) {
				name2 = ((IEditorReference)e2).getTitle();
			}
			if (e1 instanceof String) {
				name1 = (String) e1;
			}
			if (e2 instanceof String) {
				name2 = (String) e2;
			}
		}
		return name1.compareToIgnoreCase(name2);
	}
	
	public void changeSortFile() {
		fileSort++;
		fileSort = fileSort % 3;
	}
	
	public void changeSortProject() {
		projectSort++;
		projectSort = projectSort % 3;
	}
	
	public int getSortModusFile() {
		return fileSort;
	}
	
	public void setSortModusFile(int sortModus) {
		fileSort = sortModus;
	}
	
	public int getSortModusProject() {
		return projectSort;
	}
	
	public void setSortModusProject(int sortModus) {
		projectSort = sortModus;
	}
	
	public void init(IMemento memento) {
		if (memento != null && memento.getInteger("COL_FILE_SORT") != null) {
			fileSort = memento.getInteger("COL_FILE_SORT").intValue();
		}
		if (memento != null && memento.getInteger("COL_PROJECT_SORT") != null) {
			projectSort = memento.getInteger("COL_PROJECT_SORT").intValue();
		}
	}
	
	public void saveState(IMemento memento) {
		memento.putInteger("COL_FILE_SORT", fileSort);
		memento.putInteger("COL_PROJECT_SORT", projectSort);
	}
	
}