/**
 * 
 */
package net.keil.openFileView.views;

import net.keil.openFileView.OpenFileViewPlugin;
import net.keil.openFileView.Resources;
import net.keil.openFileView.preferences.PreferenceConstants;
import net.keil.openFileView.util.OpenFileItem;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.model.WorkbenchLabelProvider;

/**
 * 
 * @author Torsten Keil
 * @since 1.2.0
 */
class ViewLabelProvider extends WorkbenchLabelProvider implements ITableLabelProvider, IColorProvider {
	/**
	 * 
	 */
	private final OpenFileView view;

	/**
	 * @param view
	 */
	ViewLabelProvider(OpenFileView view) {
		this.view = view;
	}

	public String getColumnText(Object obj, int index) {
		if (obj == null) return "";
		String retValue = Resources.getString("UnknownFileLabel");
		if (obj instanceof OpenFileItem) {
			if (index == view.getFileColumnIndex()) {
				retValue = ((OpenFileItem)obj).getFileName();
			} else if (index == view.getProjectColumnIndex()) {
				retValue = ((OpenFileItem)obj).getProjectName();
			} else {
				retValue = "";
			}
		}
		if (obj instanceof Object[]) {
			if (index == view.getFileColumnIndex()) {
				retValue = ((OpenFileItem)((Object[])obj)[view.getFileColumnIndex()]).getFileName();
			} else if (index == view.getProjectColumnIndex()) {
				retValue = ((OpenFileItem)((Object[])obj)[view.getProjectColumnIndex()]).getProjectName();
			} else {
				retValue = "";
			}
		}
		return retValue;
	}
	
	public Image getColumnImage(Object obj, int index) {
		if (index == view.getFileColumnIndex()) {
			IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
			if (!store.getBoolean(PreferenceConstants.P_SHOW_FILE_ICONS)) {
				return null;
			}
			if (obj instanceof OpenFileItem) {
				return getImage(((OpenFileItem)obj).getFile());
//				return ((OpenFileItem)obj).getFileIcon(); // it's cached :-(
			} else if (obj instanceof Object[]) {
				return getImage(((OpenFileItem)((Object[])obj)[index]).getFile());
			} else {
				return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_ELEMENT);
			}
		} if (index == view.getProjectColumnIndex()) {
			IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
			if (!store.getBoolean(PreferenceConstants.P_SHOW_PROJECT_ICONS)) {
				return null;
			}
			if (obj instanceof OpenFileItem) {
				return getImage(((OpenFileItem)obj).getProject());
			} else if (obj instanceof Object[]) {
				return getImage(((OpenFileItem)((Object[])obj)[index]).getProject());
			} else {			
				return null;
			}
		} if (index == view.getCloseColumnIndex()) {
//			if (obj instanceof Object[]) {
//				int idx = ((Integer)((Object[])obj)[index]).intValue();
//				if (idx == 0) return view.closeImg;
//			}
			return view.closeImg;
		} else {
			return null;
		}
	}
	
	public Color getForeground(Object element) {
		IPreferenceStore store = OpenFileViewPlugin.getDefault().getPreferenceStore();
		String look = store.getString(PreferenceConstants.P_USE_31_LOOK);
		if (PreferenceConstants.O_31_LOOK.equals(look)) {
			return this.view.hoverColor;
		}
		if (element instanceof OpenFileItem) {
			OpenFileItem item = (OpenFileItem) element;
			if (this.view.activeEditorRef == null) { // not initialized
				IEditorPart activeEditorPart = this.view.getSite().getPage().getActiveEditor();
				IEditorPart editorPart = item.getEditorPart();
				if (editorPart != null && editorPart.equals(activeEditorPart)) {
					return this.view.hoverColor;
				} else {
					return this.view.defaultColor;
				}
			} else {
				if (item.getEditorRef().equals(this.view.activeEditorRef)) {
					return this.view.hoverColor;
				} else {
					return this.view.defaultColor;
				}
			}
		} else if (element instanceof Object[]) {
			OpenFileItem item = (OpenFileItem) ((Object[])element)[view.getFileColumnIndex()];
			if (this.view.activeEditorRef == null) { // not initialized
				IEditorPart activeEditorPart = this.view.getSite().getPage().getActiveEditor();
				IEditorPart editorPart = item.getEditorPart();
				if (editorPart != null && editorPart.equals(activeEditorPart)) {
					return this.view.hoverColor;
				} else {
					return this.view.defaultColor;
				}
			} else {
				if (item.getEditorRef().equals(this.view.activeEditorRef)) {
					return this.view.hoverColor;
				} else {
					return this.view.defaultColor;
				}
			} 

		} else {
			return this.view.defaultColor;
		}
	}
}